""" Wrapper class for Celery tasks """
import logging
import tempfile
from pathlib import Path

from celery import shared_task

import one_api.settings_prod as settings
from articles.data_extraction.crop_meta import extract_infos
from articles.models import Files

logger = logging.getLogger(__name__)


@shared_task
def extract_information_from_file(article_file_id):
    """ extract all information from a file """
    article_file = Files.objects.get(pk=article_file_id)
    logger.debug('starting task \'extract_information_from_file\' with %s', article_file)
    with tempfile.TemporaryDirectory() as temp_dirname:
        path_str = settings.MEDIA_ROOT + str(article_file.upload_file)
        path = Path(path_str)
        if path.is_file():
            extract_infos(article_file, temp_dirname, settings.MEDIA_ROOT + '/')
        else:
            logger.error('file %s does not exist', path_str)

    logger.debug('stop task \'extract_information_from_file\' with %s', article_file)
