""" All Tasks for articles """
import logging
import os

from celery import shared_task
from django.utils import timezone

from articles.data_extraction.tasks import extract_information_from_file
from articles.models import Files, Article
from one_api.settings_prod import MEDIA_ROOT

logger = logging.getLogger(__name__)


@shared_task
def celery_idle_job(number):
    """ Just a celery task to configure celery right"""
    logger.info("started celery idle task %d", number)
    counting_number = 0
    for i in range(0, 10000):
        counting_number = counting_number + i
    logger.info("idle task %d ist fertig: %d", number, counting_number)


@shared_task
def read_all_media_files():
    """ read all file in the media folder """
    media_dir = os.listdir(MEDIA_ROOT)
    logger.info("read out %i files from media root", len([name for name in media_dir if name.endswith('pdf')]))
    for m_file in list(filter(lambda file_name: file_name.endswith('pdf'), media_dir)):
        if Files.objects.filter(upload_file=m_file).count() == 0:
            file = Files(upload_file=m_file, upload_date=timezone.now())
            file.save()
        elif Files.objects.filter(upload_file=m_file, extraction_complete=False):
            logger.debug("file %s will be re-read", m_file)
            file = Files.objects.get(upload_file=m_file)
            Article.objects.filter(source_file=file).delete()
            extract_information_from_file.delay(file.pk)
        else:
            logger.debug("file %s already exists and was correct extracted in db", m_file)
