""" Test class for wizard """
import logging

from django.test import TransactionTestCase, TestCase

from articles.models import User
from articles.wizard.manipulators import create_users_in_db, deactivate_user

logger = logging.getLogger(__name__)


class OverviewView(TransactionTestCase):
    """ Test the overview view """

    def test_plain_context_data(self):
        """ test context data without DB Data """
        logger.info("started OverviewView.test_context_data")
        response = self.client.get('/wizard/overview')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['articles']['count'], 0)
        self.assertEqual(response.context['media']['count'], 0)
        self.assertEqual(response.context['tags']['count'], 0)


class HomeView(TestCase):
    """ Test the home view """

    def test_plain_context_data(self):
        """ test context data """
        logger.info("started HomeView.test_plain_context_data")
        response = self.client.get('/')
        self.assertEqual(response.status_code, 200)
        self.assertTrue(len(response.context['site_urls']['articles']) > 0)
        self.assertTrue(len(response.context['site_urls']['pr_upload']) > 0)
        self.assertTrue(len(response.context['site_urls']['wizard']) > 0)
        self.assertTrue(len(response.context['site_urls']['projekt']) > 0)


class Manipulator(TestCase):
    """ Tests methods in manipulators """

    def test_create_user(self):
        """ tests user creation """
        logger.info("started Manipulator.test_create_user")
        create_users_in_db(1)
        self.assertEqual(User.objects.all().count(), 1)
        create_users_in_db(3)
        self.assertEqual(User.objects.all().count(), 4)

    def test_deactivate_one_user(self):
        """ tests how one user can be deactivated"""
        logger.info("started Manipulator.test_deactivate_one_user")
        ids = create_users_in_db(1)
        user = User.objects.get(id_tag=ids[0])
        deactivate_user(user.id)
        self.assertEqual(User.objects.filter(activated=False).count(), 1)
        self.assertEqual(User.objects.filter(activated=True).count(), 0)

    def test_deactivate_multiple_user(self):
        """ tests how users can be deactivated"""
        logger.info("started Manipulator.test_deactivate_multiple_user")
        create_users_in_db(3)
        user_ids = []
        for user in User.objects.all():
            user_ids.append(user.id)
        deactivate_user(*user_ids)
        self.assertEqual(User.objects.filter(activated=False).count(), 3)
        self.assertEqual(User.objects.filter(activated=True).count(), 0)


class SimulatorView(TestCase):
    """ Testclass for the SimulatorView """

    def test_plain_view(self):
        """ tests the plain view """
        logger.info("started SimulatorView.test_plain_view")
        response = self.client.get('/wizard/simulator')
        self.assertEqual(response.status_code, 200)
