""" Webcropper for uni-potsdam.de/nachrichten """
# TESTED 15th of November 2021
# !!RUNTIME 4h 19min

import os
import logging
import time
import requests as rq
from bs4 import BeautifulSoup as bsx

from cropping.models import Category, Article

logger = logging.getLogger(__name__)


# helper
def get_scoop(rq_url, rq_local):
    """ request and souping for either local or web pages """
    if rq_local:
        res = ""
        with open(rq_url, 'r', encoding='UTF-8') as page:
            res += page.read()
        return bsx(res, 'html.parser')
    res = rq.get(rq_url)
    return bsx(res.text, 'html.parser')


def get_categories(branch_prefix, branch_sets):
    """ getting categories """
    # setup
    logger.info("starting the cropper")
    ls_curls = []

    # squeeze available categories
    logger.debug("squeeze available categories:")
    scoop = get_scoop(branch_prefix + str(1) + branch_sets + str(1), False)
    c_urls = list(map(lambda x: x['href'], scoop.findAll('a')))  # getting the category URLs
    c_urls = list(filter(lambda x: x.startswith("/de/nachrichten/page?tx_news_pi"), c_urls))
    c_names = list(filter(lambda x: x.text.startswith(" "), scoop.findAll('a')))  # getting names
    for url, name in zip(c_urls, c_names):
        ls_curls.append((url[70:], name.text[1: len(name.text) - 1]))
        cat_name = str(name.text[1: len(name.text) - 1])
        cat, created = Category.objects.update_or_create(name=cat_name)
        if created:
            logger.debug('saved category %s (%s)', cat.name, cat.pk)

    logger.debug(str(len(ls_curls)) + " categories found")
    return ls_curls


def run_crop(categories, branch_prefix, branch_sets, art_local, crop_session):
    """ processing categories """
    wait_index = 0
    for dv_url in categories[0:]:
        logger.debug("gathering on topic: " + dv_url[1])
        ls_urls, branch_page = [], 1

        # getting relevant URLs
        while True:
            logger.debug("getting available links on page " + str(branch_page))
            branch_url = branch_prefix + str(branch_page) + branch_sets + dv_url[0]
            if art_local:
                scoop = get_scoop(dv_url[0], True)
                t_urls = map(str, scoop.findAll('a'))
                t_urls = list(filter(lambda x: x.startswith("<a href"), t_urls))
                t_urls = list(filter(lambda x: "html" in x, t_urls))
                t_urls = list(map(lambda x: x.split('href="', 1)[1], t_urls))
                t_urls = list(map(lambda x: x.split('"', 1)[0], t_urls))
                t_urls = list(filter(lambda x: x.startswith("/"), t_urls))
                t_urls = list(dict.fromkeys(t_urls))
                url_prefix = os.getcwd()
            else:
                scoop = get_scoop(branch_url, False)
                t_urls = list(map(lambda x: x['href'], scoop.findAll('a')))
                t_urls = list(filter(lambda x: x.startswith("/de/nachrichten/detail"), t_urls))
                url_prefix = 'https://www.uni-potsdam.de'
            if not t_urls:
                break
            t_last = ""
            for t_url in t_urls:
                if t_last != t_url:
                    ls_urls.append(url_prefix + t_url)
                t_last = t_url
            branch_page += 1
            if art_local:
                break

        # work with found articles
        logger.debug("\nstarting to read the found articles")
        wait_index = read_articles(ls_urls, dv_url, wait_index, art_local, crop_session)


def read_articles(articles, category, wait_index, art_local, crop_session):
    """ read and process articles """
    for url in articles:
        logger.info("now reading: " + url)

        # respectfully waiting if necessary
        if wait_index > 200:
            logger.debug("respectfully waiting for 1 minutes...")
            time.sleep(60*int(not art_local))
            wait_index = 0
        else:
            wait_index += 1

        # compiling article
        scoop, s_article = get_scoop(url, art_local), ""
        title = scoop.find('h1')
        paragraphs = scoop.findAll('p')
        for paragraph in paragraphs[1:]:
            s_article += paragraph.text + "\\n"
        article, _ = Article.objects.update_or_create(title=title.text.strip(), session_id=crop_session.id,
                                                      defaults={'text': s_article})
        logger.debug("article saved: %i", article.id)
        s_article = title.text + "\\n\\n" + s_article
        cat_obj, _ = Category.objects.update_or_create(name=str(category[1]))
        logger.debug("added category %s to article %i", cat_obj.name, article.id)
        article.categories.add(cat_obj)
        article.save()

        # write to csv
        with open("export_raw.csv", "a", encoding='UTF-8') as tch:
            tch.write(category[1] + "," + s_article + "\n")
    return wait_index


def join_categories():
    """ join duplicated articles from different categories """
    logger.info("\nsearching & joining duplicates...")
    with open("export_raw.csv", "r", encoding='UTF-8') as rfile:
        lines = list(map(lambda x: x.split(',', 1), rfile.readlines()))
        for j, line in enumerate(lines):
            i = j + 1
            while i < len(lines):
                if line[1] == lines[i][1]:
                    logger.debug("joining duplicate between \"" + line[0] + "\" and \"" + lines[i][0] + "\"")
                    line[0] += ';' + lines[i][0]
                    lines.pop(i)
                else:
                    i += 1

    # rewrite joined information as updated csv
    logger.info("\nwriting the final .csv file...")
    with open("export.csv", "a", encoding='UTF-8') as rwr:
        for line in lines:
            rwr.write(line[0] + ',' + line[1])


def start_crop(prefix, sets, crop_session):
    """ runs the entire cropping process """
    run_crop(get_categories(prefix, sets), prefix, sets, False, crop_session)
    join_categories()
