""" Test class """

import logging
from os import listdir, remove

from django.utils import timezone
from django.test import TransactionTestCase

from cropping.models import CropSession
from cropping.cropper import read_articles, get_categories, get_scoop, run_crop, join_categories

logger = logging.getLogger(__name__)

BRANCH = 'https://www.uni-potsdam.de/de/nachrichten/page-'
SETS = '?tx_news_pi1[overwriteDemand][categories]='


def rm_if_exists(target):
    """ removes a file if found in directory """
    if target in listdir("."):
        remove(target)


class WebMediaCropper(TransactionTestCase):
    """ Test the mediacropping of webarticles """

    def test_get_scoop(self):
        """ test if beautiful soup scoop was produced """
        logger.info("started test_get_scoop")
        scoop = get_scoop("https://www.uni-potsdam.de/de/nachrichten/", False)
        self.assertTrue(scoop is not None)

    def test_get_categories(self):
        """ test if all anticipated categories have been found """
        logger.info("started test_get_categories")
        categories = get_categories(BRANCH, SETS)
        self.assertTrue(len(categories) > 0)

    def test_run_crop(self):
        """ test if the cropping has been successful """
        logger.info("started test_run_crop")
        categories = [('articles/testfiles/articles.html', 'Ethik')]
        rm_if_exists("export_raw.csv")
        crop_session = CropSession(date_of_crop=timezone.now())
        crop_session.save()
        run_crop(categories, BRANCH, SETS, True, crop_session)
        target, result = [], []
        with open("export_raw.csv", "r", encoding='UTF-8') as test:
            result = test.readlines()
        with open("articles/testfiles/raw_test.csv", "r", encoding='UTF-8') as goal:
            target = goal.readlines()
        self.assertEqual(target, result)

    def test_read_articles(self):
        """ test if articles have been read correctly """
        logger.info("started test_read_articles")
        rm_if_exists("export_raw.csv")  # TODO: do this after the deed
        articles = ["articles/test_alpen.html"]
        crop_session = CropSession(date_of_crop=timezone.now())
        crop_session.save()
        wait_index = 0
        wait_index = read_articles(articles, ("null", "Ethik"), wait_index, True, crop_session)
        self.assertEqual(wait_index, 1)
        target, result = "", ""
        with open("export_raw.csv", "r", encoding='UTF-8') as test:
            result = test.readlines()[0]
        with open("articles/test_result.csv", "r", encoding='UTF-8') as goal:
            target = goal.readlines()[0]
        self.assertEqual(target, result)
        # TODO: test with wait index over 200

    def test_join_categories(self):
        """ test if all duplicated categories have been joined """
        logger.info("started test_join_categories")
        rm_if_exists("export_raw.csv")
        rm_if_exists("export.csv")
        with open("export_raw.csv", "a", encoding='UTF-8') as wtest:
            wtest.write("Sabretoothtigerism,Once upon a time\n")
            wtest.write("Woolspinnerism,Happily ever after\n")
            wtest.write("Lannisterism,Happily ever after\n")
            wtest.write("Loneliness,I will be alone\n")
            wtest.write("Goblin Studies,Once upon a time\n")
        join_categories()
        result = []
        target = ["Sabretoothtigerism;Goblin Studies,Once upon a time\n",
                  "Woolspinnerism;Lannisterism,Happily ever after\n",
                  "Loneliness,I will be alone\n"]
        with open("export.csv", "r", encoding='UTF-8') as rtest:
            result = rtest.readlines()
        self.assertEqual(target, result)
